var _excluded = ["portal", "onHide", "onRecord", "maxDuration", "children"];

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { forwardRef, useContext, useRef, useImperativeHandle, useEffect, useState } from 'react';
import { createPortal } from 'react-dom';
import Context from './../Context/instance.js';
import CameraRecorder from './CameraRecorder.js';
var Camera = forwardRef(function (_ref, ref) {
  var portal = _ref.portal,
      onHide = _ref.onHide,
      onRecord = _ref.onRecord,
      maxDuration = _ref.maxDuration,
      children = _ref.children,
      others = _objectWithoutPropertiesLoose(_ref, _excluded);

  // context
  var context = useContext(Context) || {};

  var locale = context.locale || function (remark) {
    return remark || '';
  };

  var _useState = useState({}),
      successResult = _useState[0],
      setSuccessResult = _useState[1];

  var videoRef = useRef(null);
  var startRef = useRef(null);
  var stopRef = useRef(null);
  var timeRef = useRef(null);
  var timeTargetRef = useRef(null);
  var successRef = useRef(null);
  var errorRef = useRef(null);
  var errorTargetRef = useRef(null);
  useImperativeHandle(ref, function () {
    return videoRef.current;
  });

  function openCamera() {
    CameraRecorder.init({
      maxDuration: maxDuration,
      timer: timeTargetRef.current,
      video: videoRef.current,
      onOpened: function onOpened(e, stream) {
        if (startRef.current) startRef.current.style.display = 'block'; // e.target.play();
      },
      onStop: function onStop(e) {
        if (stopRef.current) stopRef.current.style.display = 'none';
        if (timeRef.current) timeRef.current.style.display = 'none';
        if (successRef.current) successRef.current.style.display = 'block';
        setSuccessResult(e);
      },
      onError: function onError(e, err) {
        if (errorRef.current) errorRef.current.style.display = 'block';
        if (errorTargetRef.current) errorTargetRef.current.innerHTML = err.errMsg;
      }
    });
  }

  function start() {
    CameraRecorder.startRecord();
    startRef.current.style.display = 'none';
    stopRef.current.style.display = 'block';
    timeRef.current.style.display = 'block';
  }

  function stop(e) {
    CameraRecorder.stopRecord();
  }

  function closeCamera(e) {
    CameraRecorder.stopRecord(null); // 关闭不走停止onStop回调

    if (onHide) onHide(e);
  }

  function success() {
    onRecord(successResult);
  }

  useEffect(function () {
    openCamera();
  }, []);
  return createPortal( /*#__PURE__*/React.createElement("div", _extends({
    ref: ref
  }, others, {
    className: "camera" + (others.className ? ' ' + others.className : '')
  }), /*#__PURE__*/React.createElement("video", {
    ref: videoRef,
    width: "100%",
    height: "100%",
    autoPlay: "",
    playsInline: true,
    muted: true
  }), /*#__PURE__*/React.createElement("div", {
    ref: startRef,
    className: "camera-recorder-start",
    onClick: start
  }, /*#__PURE__*/React.createElement("div", {
    className: "camera-recorder-start-round"
  }, /*#__PURE__*/React.createElement("div", {
    className: "camera-recorder-start-dot"
  }))), /*#__PURE__*/React.createElement("div", {
    ref: stopRef,
    className: "camera-recorder-stop",
    onClick: stop
  }, /*#__PURE__*/React.createElement("div", {
    className: "camera-recorder-stop-dot"
  })), /*#__PURE__*/React.createElement("div", {
    ref: successRef,
    className: "camera-success",
    onClick: success
  }, /*#__PURE__*/React.createElement("div", {
    className: "camera-success-icon"
  })), /*#__PURE__*/React.createElement("div", {
    ref: timeRef,
    className: "camera-recorder-timer"
  }, /*#__PURE__*/React.createElement("span", {
    className: "camera-recorder-timer-dot"
  }), /*#__PURE__*/React.createElement("span", {
    ref: timeTargetRef,
    className: "camera-recorder-timer-label"
  }, "00:00")), /*#__PURE__*/React.createElement("div", {
    ref: errorRef,
    className: "camera-recorder-error"
  }, /*#__PURE__*/React.createElement("span", {
    ref: errorTargetRef,
    className: "camera-recorder-error-label"
  })), /*#__PURE__*/React.createElement("div", {
    className: "camera-close",
    onClick: closeCamera
  }), children), portal || context.portal || document.getElementById('root') || document.body);
});
export default Camera;